#!/usr/local/bin/ruby -w

#    tracker2ay: convert Spectrum tracker files to Spectrum native-code formats
#    Copyright (C) 2008 Matt Westcott
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License version 3 (only)
#    as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#    Author: Matt Westcott <matt@west.co.tt>
#    14 Daisy Hill Drive, Adlington, Chorley, Lancs, PR6 9NE. UNITED KINGDOM

require 'zxfile'
require 'zxtracker'

def convert(infile, outfile)
	module_class = case infile
		when /\.stc$/ then ZXTracker::SoundtrackerModule
		when /\.sqt$/ then ZXTracker::SQTrackerModule
		when /\.pt3$/ then ZXTracker::ProTracker3Module
	end
	if module_class.nil?
		STDERR.puts "Unknown input file type: #{infile}"
		return
	end
	
	mod = module_class.open oldlocation(infile)
	
	case outfile
		when /\.tap$/ then mod.write(:tap, oldlocation(outfile))
		when /\.tzx$/ then mod.write(:tzx, oldlocation(outfile))
		when /\.ay$/ then mod.write(:ay, oldlocation(outfile))
		else STDERR.puts "Unknown output file type: #{outfile}"
	end
end

if ARGV.size == 2
	convert(*ARGV)
else
	puts "tracker2ay: Convert a Spectrum tracker file to an .ay or .tap file."
	puts "Usage: tracker2ay infile outfile"
end
