module ZXFile
	module BasicBuilder
		KEYWORDS = {
			:rnd => 0xa5,
			:inkeys => 0xa6,
			:pi => 0xa7,
			:fn => 0xa8,
			:point => 0xa9,
			:screens => 0xaa,
			:attr => 0xab,
			:at => 0xac,
			:tab => 0xad,
			:vals => 0xae,
			:code => 0xaf,
			:val => 0xb0,
			:len => 0xb1,
			:sin => 0xb2,
			:cos => 0xb3,
			:tan => 0xb4,
			:asn => 0xb5,
			:acs => 0xb6,
			:atn => 0xb7,
			:ln => 0xb8,
			:exp => 0xb9,
			:int => 0xba,
			:sqr => 0xbb,
			:sgn => 0xbc,
			:abs => 0xbd,
			:peek => 0xbe,
			:in => 0xbf,
			:usr => 0xc0,
			:strs => 0xc1,
			:chrs => 0xc2,
			:not => 0xc3,
			:bin => 0xc4,
			:or => 0xc5,
			:and => 0xc6,
			:lte => 0xc7,
			:gte => 0xc8,
			:neq => 0xc9,
			:line => 0xca,
			:then => 0xcb,
			:to => 0xcc,
			:step => 0xcd,
			:deffn => 0xce,
			:cat => 0xcf,
			:format => 0xd0,
			:move => 0xd1,
			:erase => 0xd2,
			:open => 0xd3,
			:close => 0xd4,
			:merge => 0xd5,
			:verify => 0xd6,
			:beep => 0xd7,
			:circle => 0xd8,
			:ink => 0xd9,
			:paper => 0xda,
			:flash => 0xdb,
			:bright => 0xdc,
			:inverse => 0xdd,
			:over => 0xde,
			:out => 0xdf,
			:lprint => 0xe0,
			:llist => 0xe1,
			:stop => 0xe2,
			:read => 0xe3,
			:data => 0xe4,
			:restore => 0xe5,
			:new => 0xe6,
			:border => 0xe7,
			:continue => 0xe8,
			:dim => 0xe9,
			:rem => 0xea,
			:for => 0xeb,
			:goto => 0xec,
			:gosub => 0xed,
			:input => 0xee,
			:load => 0xef,
			:list => 0xf0,
			:let => 0xf1,
			:pause => 0xf2,
			:next => 0xf3,
			:poke => 0xf4,
			:print => 0xf5,
			:plot => 0xf6,
			:run => 0xf7,
			:save => 0xf8,
			:randomize => 0xf9,
			:if => 0xfa,
			:cls => 0xfb,
			:draw => 0xfc,
			:clear => 0xfd,
			:return => 0xfe,
			:copy => 0xff
		}

		def self.build(program_lines)
			program = ''
			for line_number in program_lines.keys.sort
				line = ''
				for token in program_lines[line_number]
					line << token.to_spectrum_basic
				end
				line << "\x0d"
				program << ([line_number, line.size].pack('nv') + line)
			end
			program
		end
	end
end

class Symbol
	def to_spectrum_basic
		ZXFile::BasicBuilder::KEYWORDS[self].chr
	end
end

class String
	def to_spectrum_basic
		self.gsub("\xa3", "\x60").gsub("\xa9", "\x7f")
	end
end

class Fixnum
	def to_spectrum_basic
		case self
			when 0..0xffff
				self.to_s + [0x0e, 0, 0, self, 0].pack('cccvc')
			when -0xffff..-1
				self.to_s + [0x0e, 0, 0xff, self + 0x10000, 0].pack('cccvc')
			else
				raise "Numbers outside of range (-65535..65535) not implemented yet"
		end
	end
end
