module ZXFile
	module Tape
		class RomTapeBlock

			attr_accessor :data, :flag

			def initialize(opts = {})
				@data = opts[:data]
				@flag = opts[:flag] || 0xff
			end
			
			def trailing_pause_length # in ms
				1000
			end
			
			def checksum
				sum = self.flag
				data = self.data
				data.each_byte do |b|
					sum ^= b
				end
				sum
			end
			
			def raw_data
				self.flag.chr + self.data + self.checksum.chr
			end
			
			def write(file_type, file)
				case file_type
					when :tap then write_tap(file)
					when :tzx then write_tzx(file)
					else raise "Unknown filetype"
				end
			end

			def write_tap(file)
				rd = self.raw_data
				ZXFile::TAPFile.write(file) do |io|
					io.write [rd.size].pack('v')
					io.write rd
				end
			end
			
			def write_tzx(file)
				rd = self.raw_data
				ZXFile::TZXFile.write(file) do |io|
					io.write [0x10, self.trailing_pause_length, rd.size].pack('cvv')
					io.write rd
				end
			end
		end
		
		class CodeFileHeader < RomTapeBlock
			attr_accessor :name, :start_address, :length
			def initialize(opts = {})
				@name = opts[:name]
				@start_address = opts[:start_address]
				@length = opts[:length]
				super
			end
			
			def flag
				0x00
			end
			
			def data
				[0x03, @name, @length, @start_address, 0x8000].pack 'cA10vvv'
			end
		end

		class BasicFileHeader < RomTapeBlock
			attr_accessor :name, :start_line, :program_length, :length
			def initialize(opts = {})
				@name = opts[:name]
				@start_line = opts[:start_line]
				@program_length = opts[:program_length]
				@length = opts[:length]
				super
			end
			
			def flag
				0x00
			end
			
			def data
				[0x00, @name, @length, @start_line || 0x8000, @program_length].pack 'cA10vvv'
			end
		end
		
	end
end
