require 'zxfile'
require 'ayfile'

module ZXTracker
	class ProTracker3Module
		def self.open(filename)
			self.new( :data => (File.open(filename, 'rb') {|f| f.read}) )
		end
		
		def self.player
			@@player ||= File.open(
				File.join(File.dirname(__FILE__), 'pt3_player.bin'), 'rb') {|f| f.read}
		end
		
		attr_accessor :data
		def initialize(opts = {})
			@data = opts[:data]
		end
		
		def name
			@data[0x1e..0x3d].strip
		end
		
		def author
			@data[0x42..0x61].strip
		end
		
		def write(format, file)
			if format == :ay
				ayfile = AYFile.new(:author => self.author,
					:misc => 'created by tracker2ay', :songs => [
					AYFile::Song.new(:name => self.name,
						:stack => 0xbfff, :init => 0xc000, :inter => 0xc005,
						:blocks => [
							AYFile::MemoryBlock.new(0xc000, ZXTracker::ProTracker3Module.player),
							AYFile::MemoryBlock.new(0xc86e, @data),
						])
					])
				ayfile.write(file)
			else
				tape = ZXFile::Medium.new
				tape << ZXFile::BasicFile.new(
					:name => 'pt3_player',
					:start_line => 10,
					:program => {
						10 => [:clear, 0xbfff],
						20 => [:load, '"pt3_player"', :code],
						30 => [:load, '"pt3_data"', :code],
						40 => [:randomize, :usr, 0xc000],
						50 => [:pause, 1, ':', :randomize, :usr, 0xc005, ':', :goto, 50]
				})
				tape << ZXFile::CodeFile.new(
					:name => 'pt3_player',
					:start_address => 0xc000,
					:data => ZXTracker::ProTracker3Module.player)
				tape << ZXFile::CodeFile.new(
					:name => 'pt3_data',
					:start_address => 0xc86e,
					:data => @data)
				tape.write(format, file)
			end
		end
	end
end
