require 'zxfile'

module ZXTracker
	class SQTrackerModule
		def self.open(filename)
			self.new( :data => (File.open(filename, 'rb') {|f| f.read}),
				:name => File.basename(filename) )
		end
		
		def self.player
			@@player ||= File.open(
				File.join(File.dirname(__FILE__), 'sqt_player.bin'), 'rb') {|f| f.read}
		end
		
		attr_accessor :data, :name
		def initialize(opts = {})
			@data = opts[:data]
			@name = opts[:name]
		end
		
		def relocate(new_address)
			header = @data[0..13].unpack('v*')
			base_address = header[1] - 10
			offset = new_address - base_address
			relocation_end = header[6] - base_address
			@data[0..1] +
				@data[2...relocation_end].unpack('v*').collect {|i| i+offset}.pack('v*') +
				@data[relocation_end..-1]
		end
		
		def data_with_player
			ZXTracker::SQTrackerModule.player + self.relocate(0xc000 + ZXTracker::SQTrackerModule.player.size)
		end
		
		def write(format, file)
			if format == :ay
				ayfile = AYFile.new(:misc => 'created by tracker2ay', :songs => [
					AYFile::Song.new(:name => (self.name || '(unknown)'),
						:stack => 0xbfff, :init => 0xc000, :inter => 0xc030,
						:blocks => [
							AYFile::MemoryBlock.new(0xc000, data_with_player)
						])
					])
				ayfile.write(file)
			else
				tape = ZXFile::Medium.new
				tape << ZXFile::BasicFile.new(
					:name => 'sqt_player',
					:start_line => 10,
					:program => {
						10 => [:clear, 0xbfff],
						20 => [:load, '"sqt_data"', :code],
						30 => [:randomize, :usr, 0xc000],
						40 => [:pause, 1, ':', :randomize, :usr, 0xc030, ':', :goto, 40]
				})
				tape << ZXFile::CodeFile.new(
					:name => 'sqt_data',
					:start_address => 0xc000,
					:data => data_with_player)
				tape.write(format, file)
			end
		end
	end
end
